//By L4ggcrafter
import { world, system, ItemStack} from '@minecraft/server'
import {VECTOR} from "./mathUtils.js"
import {shoot} from "./gunFunctions/shootProjectile.js"
import {playEffects} from "./gunFunctions/specialEffects.js"
import {setActionBar} from "./gunFunctions/actionBar.js"
import {updateAmmo} from "./gunFunctions/ammoSystem.js"
import {finishReloading} from "./gunFunctions/reloadSystem.js"
import {setMovement} from "./gunFunctions/movement.js"
import {ReloadTypes} from "./gunFunctions/reloadSystem.js"
import {PlayerEquipment} from "./playerComponents.js"
import {onGunStopUse} from "./itemUse.js"
import {Guns} from "./Guns.js"
import {initializePlayerData, PlayerGunsData, setUpGunsData} from "./PlayerGunsData.js"

system.runInterval(() => {
    let players = world.getAllPlayers()
    for (let player of players) {
        initializePlayerData (player)
        
        if (!isPlayerAlive (player)) {
            onGunUserDead (player)
            continue
        }
        
        const item = PlayerEquipment.getItemHold(player)
        if (!item) continue
        initializeGun (player, item)
        finishReloading(player, item)
        if (!Guns.hasOwnProperty(item.typeId)) {
            if (!ReloadTypes.hasOwnProperty(item.typeId)) continue
            setActionBar(player)
            setUpEmptyGun (player, item)
            continue
        }
        gunLoop (player, item)
        setActionBar(player)
        setMovement(player, item)
        

    }
}, 0)
function setUpEmptyGun(player, item) {
    let gunId = item.getDynamicProperty("L4gg-gunId")
    if (!gunId) return
    setUpGunsData (player, item, false, player.selectedSlotIndex) 
}
function gunLoop (player, item) {
    if (!PlayerGunsData.hasOwnProperty(player.id)) {
        //initializePlayerData(player)
        return
    }
    updateGunsCooldown(player.id)
    const gunId = item.getDynamicProperty("L4gg-gunId")
    if (!gunId) {
        return
    }
    initializePlayerGunsData(player, item, gunId)
    const GunId = item.getDynamicProperty("L4gg-gunId") 
    let isNotUsingGun = !PlayerGunsData[player.id].isUsing
    if (isNotUsingGun) return
    const GunHoldData = PlayerGunsData[player.id].guns[gunId]
    /*
    const gunIndex = PlayerGunsData[player.id].guns[gunId].index
    if (gunIndex !== player.selectedSlotIndex) {
        onGunStopUse (player, item) 
        console.warn("changed")
        return 
    }
    */
    
    if (!Guns[item.typeId]) {
        return 
    }
    if (GunHoldData.currentCooldown > 0) return
    shoot(player, item)
    playEffects(player, item)
    addCooldown (player.id, item.typeId, GunId)
    updateAmmo(player, item, GunHoldData)
}
function initializeGun (player, item) {
    let gunId = item.getDynamicProperty("L4gg-gunId")
    if (gunId) return
    if (!Guns.hasOwnProperty(item.typeId) && !ReloadTypes.hasOwnProperty(item.typeId)) return
    gunId = setUpGunId(player, item)
    let ammo = Guns[item.typeId]?.maxAmmo || 0
    item.setDynamicProperty("L4gg-ammo", ammo)
    PlayerEquipment.setItemHold(player, item)
    initializePlayerGunsData(player, item, gunId)
}
function setUpGunId (player, item) {
    let id = world.getDynamicProperty("L4gg-gunId") || 0
    id++
    world.setDynamicProperty("L4gg-gunId", id)
    item.setDynamicProperty("L4gg-gunId", id)
    return id
}
function initializePlayerGunsData(player, item, gunId) {
    let isInitialized = PlayerGunsData[player.id].guns.hasOwnProperty(gunId)
    if (isInitialized) return
    setUpGunsData(player, item, false)
}
/*
function initializePlayerData(player) {
    const item = PlayerEquipment.getItemHold(player)
    if (item == undefined) return
    const gunId = item.getDynamicProperty("L4gg-gunId")
    if (gunId) {
        setUpGunsData(player, item, false)
    }
*/


function addCooldown (playerId, itemType, gunId) {
    let gunCoolDown = Guns[itemType].cooldown
        PlayerGunsData[playerId].guns[gunId].currentCooldown = gunCoolDown
}
function hasChangedItem (player, gunId) {
    let currentItem = PlayerEquipment.getItemHold(player)
    if (!currentItem) return true
    let currentItemId = currentItem.getDynamicProperty("L4gg-gunId")
    if (!currentItemId) return true
    return gunId !== currentItemId
}
function updateGunsCooldown(playerId) {
    const GunsData = PlayerGunsData[playerId].guns
    for (let i in GunsData) {
        if (GunsData[i].currentCooldown > 0) {
            GunsData[i].currentCooldown--
            
        }
    }
}
function onGunUserDead (player) {
    if (PlayerGunsData.hasOwnProperty(player.id) && PlayerGunsData[player.id].hasOwnProperty("guns")) {
        PlayerGunsData[player.id].isUsing = false
    }
}

function setPlayerKnockback () {
    if (system.currentTick % 2 == 0) {
        let players = world.getAllPlayers()
        players.forEach((player) => {
            player.applyKnockback(1, 1, 0.15, 0.1)
        })
    }
}
function isPlayerAlive (player) {
    const healthComp = player.getComponent("health")
    const currentHealth = healthComp.currentValue
    return currentHealth > 0
}
