import { world, system, ItemStack} from '@minecraft/server'
import {VECTOR} from "../mathUtils.js"
import {Guns} from "../Guns.js"
import {PlayerGunsData} from "../PlayerGunsData.js"
import {ReloadTypes} from "./reloadSystem.js"
import {PlayerEquipment} from "../playerComponents.js"
export function setActionBar(player) {
    const item = PlayerEquipment.getItemHold(player)
    if (item == undefined ) return
    
    let ammo = item.getDynamicProperty("L4gg-ammo") || 0
    if (Guns.hasOwnProperty(item.typeId)) {
        const maxAmmo = Guns[item.typeId].maxAmmo 
        const gunId = item.getDynamicProperty("L4gg-gunId")
        if (PlayerGunsData.hasOwnProperty(player.id)  && PlayerGunsData[player.id].isUsing) {
            const GunHoldData = PlayerGunsData[player.id].guns[gunId]
            ammo = GunHoldData.currentAmmo
        }
        let actionbar = `[ ${ammo} / ${maxAmmo} ]`
        
        actionbar += " "+getCooldown(player, item, gunId)
        //actionbar += " id "+gunId
        
        player.onScreenDisplay.setActionBar(actionbar)
        
        
        
    } else if (ReloadTypes.hasOwnProperty(item.typeId)) {
        const maxAmmo = ReloadTypes[item.typeId].ammo 
        const ammo = item.getDynamicProperty("L4gg-ammo")
        if (!ammo) {
            player.onScreenDisplay.setActionBar("[ 0 / "+maxAmmo+" ]")
            return
        }
        let reloadTime = ReloadTypes[item.typeId].reloadTime
        let reloadProgress = item.getDynamicProperty("L4gg-reloadTime")
        reloadProgress = reloadTime - Math.max(reloadProgress - system.currentTick, 0)
        let text = "Reloading "
        text += getBar(reloadProgress/2, reloadTime/2, "§2")
        //const gunId = item.getDynamicProperty("L4gg-gunId")
        //text += " id "+gunId
        player.onScreenDisplay.setActionBar(text)
    }
}

function getMaxAmmo (item) {
    if (Guns.hasOwnProperty(item.typeId)) {
        return Guns[item.typeId].reloading.maxAmmo
    } else {
        return Mags[item.typeId].reloading.maxAmmo
    }
}
function display (player, item) {
    if (Mags.hasOwnProperty(item.typeId)) {
        let ammo = item.getDynamicProperty("L4gg-ammo")
        player.onScreenDisplay.setActionBar("ammo: "+ammo)
    }
}
const CooldownColor = "§4"

function getCooldown (player, item, gunId) {
    const maxCooldown = Guns[item.typeId].cooldown
    if (PlayerGunsData.hasOwnProperty(player.id)
    && PlayerGunsData[player.id].hasOwnProperty("guns") &&
    PlayerGunsData[player.id].guns.hasOwnProperty(gunId) ) {
       let cooldown = PlayerGunsData[player.id].guns[gunId].currentCooldown || 0
       
       return getBar(cooldown, maxCooldown, CooldownColor)
    }
    return getBar(0, maxCooldown, CooldownColor)
}

function getBar (current, max, CooldownColor) {
    let cooldown = CooldownColor
    for (let i = 0; i < max; i++) {
        if (i >= current) {
            cooldown += "§7"
            
        }
        cooldown  += "▎"
    }
    return cooldown
}