import { world, system, ItemStack} from '@minecraft/server'
import {PlayerGunsData} from "../PlayerGunsData.js"
import {PlayerEquipment} from "../playerComponents.js"
import {Guns} from "../Guns.js"

export const ReloadTypes = {
    "l4gg:ak47_empty": {
        ammo: Guns["l4gg:ak47"].maxAmmo,
        convertTo: "l4gg:ak47",
        isMag: true,
        id: "l4gg:ak47_mag",
        reloadTime: 40
    },
    "l4gg:m16_empty": {
        ammo: Guns["l4gg:m16"].maxAmmo,
        convertTo: "l4gg:m16",
        isMag: true,
        id: "l4gg:m16_mag",
        reloadTime: 40
    },
    "l4gg:glock_empty": {
        ammo: Guns["l4gg:glock"].maxAmmo,
        convertTo: "l4gg:glock",
        isMag: true,
        id: "l4gg:glock_mag",
        reloadTime: 40
    },
    "l4gg:shotgun_empty": {
        ammo: Guns["l4gg:shotgun"].maxAmmo,
        convertTo: "l4gg:shotgun",
        count: 12,
        isMag: false,
        id: "l4gg:shotgun_bullet_item",
        reloadTime: 40
    },
    "l4gg:awp_empty": {
        ammo: Guns["l4gg:awp"].maxAmmo,
        convertTo: "l4gg:awp",
        isMag: true,
        id: "l4gg:awp_mag",
        reloadTime: 45
    },
}


export function finishReloading (player, item) {
    if (!item || !ReloadTypes.hasOwnProperty(item.typeId)) return
    const ammo = item.getDynamicProperty("L4gg-ammo")
    if (!ammo) return
    const reloadTime = item.getDynamicProperty("L4gg-reloadTime")
    if (reloadTime > system.currentTick) return
    const id = item.getDynamicProperty("L4gg-gunId")
    
    
    const GunTypeId = ReloadTypes[item.typeId].convertTo
    let reloadedGun = new ItemStack (GunTypeId, 1)
    reloadedGun.setDynamicProperty("L4gg-gunId", id)
    reloadedGun.setDynamicProperty("L4gg-ammo", ammo)
    
    PlayerEquipment.setItemHold(player, reloadedGun)
    
    //const slot = PlayerGunsData[player.id].guns[id].index
    //console.warn(" slot "+slot+" id "+id)
    //setItemAtSlot(player, slot, item)
    if (!PlayerGunsData[player.id].guns.hasOwnProperty(id)) {
        return
    }
    if (!PlayerGunsData[player.id].guns[id]?.currentAmmo) {
        console.warn("reloaded canceled")
        return
    }
    PlayerGunsData[player.id].guns[id].currentAmmo = ammo
}
export function reloadGun (player, item) {
    if (item && !ReloadTypes.hasOwnProperty(item.typeId)) return
    setGunId (item)
    const isReloading = item.getDynamicProperty("L4gg-isReloaded")
    if (isReloading) return
    const ReloadInfo = ReloadTypes[item.typeId]
    if (ReloadInfo.isMag) {
        let hasMag = clearItemsOfType(player, ReloadInfo.id, 1)
        if (!hasMag) {
            player.sendMessage("No Mag Found")
            return
        }
        item.setDynamicProperty("L4gg-ammo", ReloadInfo.ammo)
        item.setDynamicProperty("L4gg-reloadTime", system.currentTick + ReloadInfo.reloadTime)
        item.setDynamicProperty("L4gg-isReloaded", true)
        //const id = item.getDynamicProperty("L4gg-gunId")
        //const slot = PlayerGunsData[player.id].guns[id].index
        
        //setItemAtSlot(player, slot, item)
        PlayerEquipment.setItemHold(player, item)
    } else {
        let ammoCount = clearItemsOfType(player, ReloadInfo.id, ReloadInfo.count)
        if (!ammoCount) {
            player.sendMessage("No Bullet Found")
            return
        }
        item.setDynamicProperty("L4gg-ammo", ammoCount)
        item.setDynamicProperty("L4gg-reloadTime", system.currentTick + ReloadInfo.reloadTime)
        item.setDynamicProperty("L4gg-isReloaded", true)
        //const gunId = item.getDynamicProperty("L4gg-gunId")
        //const slot = PlayerGunsData[player.id].guns[gunId].index
        //setItemAtSlot(player, slot, item)
        PlayerEquipment.setItemHold(player, item)
        world.sendMessage("Ammo Count "+ammoCount)
    }
}
function setGunId (item) {
    let itemId = item.getDynamicProperty("L4gg-gunId")
    if (itemId) return
    let id = world.getDynamicProperty("L4gg-gunId") || 0
    id++
    world.setDynamicProperty("L4gg-gunId", id)
    item.setDynamicProperty("L4gg-gunId", id)
}

const InventorySize = 36
function clearItemsOfType(player, itemType, maxItemsToClear) {
    let clearedAmount = 0;
    let inventory = player.getComponent("minecraft:inventory").container;
    for (let i = 0; i < InventorySize; i++) {
        const item = inventory.getItem(i)
        if (item && item.typeId == itemType) {
            let itemAmount = item.amount
            let itemsToClear = Math.min(itemAmount, maxItemsToClear)
            itemAmount -= itemsToClear
            if (itemAmount <= 0) {
                inventory.setItem(i, undefined)
            } else {
                item.amount = itemAmount
                inventory.setItem(i, item)
            }
            clearedAmount += itemsToClear;
            maxItemsToClear -= itemsToClear;
            if (maxItemsToClear <= 0) {
                break;
            }
        }
    }
    return clearedAmount;
}
function setItemAtSlot(player, slot, item) {
    const inventory = player.getComponent("inventory").container
    inventory.setItem(slot, item)
}