import { world, system, ItemStack} from '@minecraft/server'
import {VECTOR} from "../mathUtils.js"
import {Guns} from "../Guns.js"
export function playEffects (player, item) {
    if (!Guns[item.typeId]) return
    playFireSound (player, item)
    spawnParticle (player, item)
    player.playAnimation("animation.gun.shoot")
    cameraShake (player, item) 
    setLight (player)
}
function spawnParticle (player, item) {
    //if (!Guns[item.typeId].hasOwnProperty("particle")) return
    const particle = Guns[item.typeId].particle.id
    const xAngle = Guns[item.typeId].particle.xAngle
    const yAngle = Guns[item.typeId].particle.yAngle
    const distance = Guns[item.typeId].particle.distance
    let velocity = player.getVelocity()
    let viewDirection = player.getViewDirection()
    let particleLocation = player.getHeadLocation()
    if (!player.isSneaking) {
        viewDirection = VECTOR.rotateXZ (viewDirection, xAngle)
        viewDirection = VECTOR.rotateVectorByPitch(viewDirection, yAngle)
    }
    viewDirection = VECTOR.multiplyByNum (viewDirection, distance)
    particleLocation = VECTOR.addWithVector(particleLocation, viewDirection)
    particleLocation = VECTOR.addWithVector(particleLocation, velocity)
    const {x, y, z} = particleLocation
    player.runCommand(`particle ${particle} ${x} ${y} ${z}`)
    
}
const light_block = "light_block_10"
function setLight (player ) {
    let location = player.getHeadLocation()
    const {x, y, z} = location
    player.runCommandAsync(`fill ${x} ${y} ${z} ${x} ${y} ${z} ${light_block} replace air`)
    system.runTimeout(() => {
        player.runCommandAsync(`fill ${x} ${y} ${z} ${x} ${y} ${z} air replace ${light_block}`)
    }, 2)
}

function playFireSound (player, item) {
    const fireSound = Guns[item.typeId].sounds.fire.id
    const volume = Guns[item.typeId].sounds.fire.volume || 1
    const pitch = Guns[item.typeId].sounds.fire.pitch || 1
    player.dimension.playSound(fireSound, player.location, {volume: volume})
}
function cameraShake (player, item) {
    if (!Guns[item.typeId].hasOwnProperty("cameraShake")) return
    const intensity = Guns[item.typeId].cameraShake.intensity
    const seconds = Guns[item.typeId].cameraShake.seconds
    player.runCommandAsync(`camerashake add @s ${intensity} ${seconds} rotational`)
}
function setScoreboard (player) {
    player.runCommandAsync("scoreboard players set @s L4gg-firing 1")
}